
:- lib(stoics_lib:kv_decompose/3).

exp_diffex_defaults( Args, Defs ) :-
    % this makes EvLog default: true
    ( (memberchk(exp_ev_log(EvLog),Args),EvLog==false) ->
        EvDefs = [  as_non(pvalue),
                    exp_ev_cnm('experiment'),
                    exp_ev_cut_let(-1),
                    exp_ev_cut_get(2),
                    exp_ev_include_inf(false),
                    exp_ev_log(false)
            ]
        ; 
        EvDefs = [  as_non(all),
                    exp_ev_cnm('log2FC'),
                    exp_ev_cut_let(-1),
                    exp_ev_cut_get(1),
                    exp_ev_include_inf(true),
                    exp_ev_log(true)
            ]
    ),
    Defs = [   
                exp_pv_cnm('adj.pvalue'),
                exp_pv_cut(0.05),
                gene_id_cnm('Symbols')
                | EvDefs
    ].

/** exp_diffex( +Exp, -DEPrs, -NonDEPrs, +Opts ).

Select a sublist of significanntly differenentially expressed genes in an experiment. Exp is in the form of a mtx/1 matrix.
Both selected (DEPrs) and rejected (NonDEPrs) are in the form of Gene-Ev, where Ev is the expression value for Gene 
in Exp. 

We use ev as short for expression value, to avoid confusion with _exp_ which short for experiment.

Opts
  * as_non(AsNon)
    what to return as non differentially expressed: either everything in Gcnm (AsNon=all, default when EvLog is true),
    or only those with numeric pvalue (AsNon=pvalue, default when EvLog is false)

  * exp_pv_cnm(ExpPcnm='adj.pvalue')
    the experimental column (found in MsF) on which Pcut is applied as a filter

  * exp_pv_cut(Pcut=0.05)
    p.value cut off for experimental input from MSstats

  * exp_ev_log(EvLog=true)
    are the expression values log values 

  * exp_ev_cnm(EvCnm)
    expession value column name (_log2FC_ if EvLog=true, _expression_ otherwise)
  
  * exp_ev_cut_let(EvLet=-1)
    expression values below (less or equal) which values are selected

  * exp_ev_cut_get(EvGet)
    expression values above (greater or equal) which values are selected (defaults is 2 if EvLog is false, and 1 otherwise)

  * exp_ev_include_inf(IncInf)
    include infinity values as diffexs ? (default is _false_ if EvLog is false, and _true_ otherwise)

  * gene_id_cnm(Gcnm='Symbols')
    column name for the key value in the pair lists: DEPrs and NonDEPrs.

==
?-  debug(testo),
    lib(debug_call),
    lib(mtx),
    D = '/home/nicos/ac/uci/proj/prcf010/totals/ms_stats-18.10.31',
    working_directory( Old, D ),
    CompF = 'ComparisonResult.normEqua.csv',
    mtx( CompF, Mtx, convert(true) ),
    debug_call( testo, dims, mtx_comp/Mtx ),
    mtx_column_values_select( Mtx, 'Label', 'D-C', Sub, _, true ),
    assert( sub(Sub) ),
    debug_call( testo, dims, mtx_sub/Sub ),
    exp_diffex( Sub, DEPrs, NonDEPrs, [] ),
    debug_call( testo, length, de/DEPrs ),
    debug_call( testo, length, nde/NonDEPrs ),
    working_directory( _, Old ).

% Dimensions for matrix,  (mtx_comp) nR: 125077, nC: 13.
% Dimensions for matrix,  (mtx_sub) nR: 5957, nC: 13.
% Length for list, de: 135.
% Length for list, nde: 5821.
true.

?- 
    sub(Sub),
    exp_diffex( Sub, DEPrs, NonDEPrs, exp_pv_cut(0.01) ),
    debug_call( testo, length, de/DEPrs ),
    debug_call( testo, length, nde/NonDEPrs ).

% Length for list, de: 60.
% Length for list, nde: 5896.
true.

?-
    sub( Sub ),
    Opts = [exp_ev_cut_let(inf),exp_ev_cut_get(-inf)],
    exp_diffex( Sub, DEPrs, NonDEPrs, Opts ),
    debug_call( testo, length, de/DEPrs ),
    debug_call( testo, length, nde/NonDEPrs ).

% Length for list, de: 344.
% Length for list, nde: 5612.
true.
==

@author nicos angelopoulos
@version  0.1 2019/04/16
*/

exp_diffex( MtxIn, DEPrs, NonDEPrs, Args ) :-
    Self = exp_diffex,
    options_append( Self, Args, Opts ),
    options( as_non(AsNon), Opts ),
    options( exp_pv_cnm(ExpPCnm), Opts ),
    options( exp_pv_cut(Pcof), Opts ),
    options( exp_ev_cnm(ExpEvCnm), Opts ),
    options( exp_ev_cut_let(EvLet), Opts ),
    options( exp_ev_cut_get(EvGet), Opts ),
    options( exp_ev_include_inf(InfInc), Opts ),
    options( gene_id_cnm(Gcnm), Opts ),
    mtx( MtxIn, Mtx, convert(true) ),
    Mtx = [Hdr|Rows],
    Cids = [ExpPCnm,ExpEvCnm,Gcnm],
    CPos = [PvPos,EvPos,GnPos],
    maplist( mtx_header_column_name_pos(Hdr), Cids, _Cnms, CPos ),
    exp_diffex_separate( Rows, PvPos, EvPos, GnPos, Pcof, EvLet, EvGet, InfInc, AsNon, DEPrs, NonDEPrs ).

exp_diffex_separate( [], _PvPos, _EvPos, _GnPos, _Pcof, _EvLet, _EvGet, _InfInc, _AsNon, [], [] ).
exp_diffex_separate( [Row|Rows], PvPos, EvPos, GnPos, Pcof, EvLet, EvGet, InfInc, AsNon, DEPrs, NonDEPrs ) :-
    arg( PvPos, Row, Pv ), 
    arg( EvPos, Row, Ev ),
    arg( GnPos, Row, Gn ),
    ( exp_diffex_select(Pv,Ev,Pcof,EvLet,EvGet,InfInc) ->
        DEPrs = [Gn-Ev|TDEPrs],
        NonDEPrs = TNonDEPrs
        ;
        DEPrs = TDEPrs,
        ( (AsNon == pvalue, \+ number(Pv)) -> NonDEPrs = TNonDEPrs; NonDEPrs = [Gn-Ev|TNonDEPrs] )
    ),
    exp_diffex_separate( Rows, PvPos, EvPos, GnPos, Pcof, EvLet, EvGet, InfInc, AsNon, TDEPrs, TNonDEPrs ).


exp_diffex_select( Pv, Ev, Pcof, EvLet, EvGet, InfInc ) :-
    number( Pv ), 
    Pv < Pcof,
    number(Ev),
    ( Ev =< EvLet ; EvGet =< Ev ),
    exp_diffex_if_inf_include( Ev, InfInc ).

exp_diffex_if_inf_include( Val, Incl ) :-
    ( Val is inf; Val is - inf ),
    !,
    Incl == true.
exp_diffex_if_inf_include( _Val, _Incl ).
